--[[
 Send information about played movie to Twitter

 Copyright © 2010 VideoLAN and AUTHORS

 Authors:  Maciej Bałuta (mbaluta@o2.pl)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
--]]

twitter_credentials = { }

-- Some global variables: widgets
dlg = nil          -- dialog
pininput = nil
tweetinput = nil
warning = nil

require("twitter_oauth")

-- Script descriptor, called when the extensions are scanned
function descriptor()
    return { title = "Twitter" ;
             version = "1.0" ;
             author = "Maciej Bałuta" ;
             url = 'http://www.twitter.com/';
             shortdesc = "Twitter";
             description = "<center><b>Twitter</b></center><br />"
                        .. "Tweets about recently played movie. ";
             capabilities = { "input-listener" } }
end

-- Returns current movie's title (removes some tags)
function get_current_title()
    local item = vlc.input.item()
    local name = item and item:name()
    if name ~= nil then
		-- look for meta-data
		local metas = item:metas()
		if metas["title"] then
			return metas["title"]
		end
		-- change some characters to space (you know, these all torrent filenames are non-human...)
        name = string.gsub(name, "(.*)(%.%w+)$", "%1")
        name = string.gsub(name, "(%.)", " ")
        name = string.gsub(name, "(%[.*%].*$)", "")
        name = string.gsub(name, "(hdtv.*)", "")
        name = string.gsub(name, "(HDTV.*)", "")
        name = string.gsub(name, "(divx.*)", "")
        name = string.gsub(name, "(DIVX.*)", "")
        name = string.gsub(name, "(DivX.*)", "")
        name = string.gsub(name, "(xvid.*)", "")
        name = string.gsub(name, "(Xvid.*)", "")
        name = string.gsub(name, "(XviD.*)", "")
        name = string.gsub(name, "(%(%d*%))", "")
        name = string.gsub(name, "(x264.*)", "")
        name = string.gsub(name, "(720p.*)", "")
        name = string.gsub(name, "(1080p.*)", "")
        name = string.gsub(name, "(dvdrip.*)", "")
        name = string.gsub(name, "(DVDRip.*)", "")
        name = string.gsub(name, "(DvdRip.*)", "")
        name = string.gsub(name, "(DVDSCR.*)", "")
        name = string.gsub(name, "(PPVRip.*)", "")
    end
    return trim(name)
end

function trim(str)
    if not str then return "" end
    return string.gsub(str, "^%s*(.*)+%s$", "%1")
end

function update_title()
	if tweetinput then
		tweetinput:set_text(get_current_title())
	end
end

-- Function called when the input (media being read) changes
function input_changed()
    update_title()
end

-- First function to be called when the extension is activated
function activate()
    check_for_config()
end

-- This function is called when the extension is disabled
function deactivate()
end

-- Called on beggining, checks for twitter credentials, if there are none, it begins configuration process
function check_for_config()
	local fh = io.open("twitter_credentials.txt")
	if not fh then
		-- begin configuration
		dlg = vlc.dialog("Twitter configuration")
		local label = dlg:add_label("Getting authorization link, please wait...", 1, 1, 3, 1)
		dlg:show()
		local url, errors = Twitter_AuthenticateNewCredentials()
		if not url then
			label:set_text("<span style=\"color:#A00\">An error occured:" .. errors .. "</span>")
		else
			label:set_text("You need to authorize this application before you can tweet. Please enter the following page,<br/> authorize this app, copy PIN, paste it here and click Continue:<br/><a href=\"" .. url .. "\">" .. url .. "</a>")
			dlg:add_label("<b>PIN</b>:", 1, 2, 1, 1)
			pininput = dlg:add_text_input("", 2, 2, 1, 1)
			dlg:add_button("Continue", click_pin, 3, 2, 1, 1)
			warning = dlg:add_label("", 1, 3, 3, 1)
		end
	else
		-- parsing file data (token, token secret, screen name and user id)
		for line in fh:lines() do 
			local key, value = line:match("(.*)=(.*)\r")
			twitter_credentials[key] = value
		end
		fh:close()
		tweet_movie_dialog()
	end
end

function click_pin()
	if pininput:get_text():match("^[0-9]+$") then
		warning:set_text("Please wait...")
		dlg:update()
		local credentials, errors = Twitter_AuthenticateNewCredentials_PIN(pininput:get_text())
		if errors then
			warning:set_text("<span style=\"color:#A00\">An error occured: " .. errors .. "</span>")
		else
			local filedata = ""
			-- Save credentials for future use
			for k, v in pairs(credentials) do
				filedata = filedata .. k .. "=" .. v .. "\r\n"
			end
			local fh = io.open("twitter_credentials.txt", "w")
			if not fh then
				warning:set_text("<span style=\"color:#A00\">An error occured: Could not make a file (read-only filesystem?)</span>")
			else
				fh:write(filedata)
				fh:close()
				twitter_credentials = credentials
				tweet_movie_dialog()
			end
		end
	else
		warning:set_text("<span style=\"color:#A00\">PIN must be a number!</span>")
	end
end

-- creates normal dialog when twitter credentials are available
function tweet_movie_dialog()
	if dlg then
		dlg:delete()
	end
	dlg = vlc.dialog("Twitter")
	dlg:add_label("Twitting as: " .. twitter_credentials.screen_name, 1, 1, 3, 1)
	dlg:add_label("<b>Movie title</b>:", 1, 2, 1, 1)
	tweetinput = dlg:add_text_input(get_current_title(), 2, 2, 1, 1)
	dlg:add_button("Tweet", click_tweet, 3, 2, 1, 1)
	warning = dlg:add_label("", 1, 3, 3, 1)
	dlg:show()
end

function click_tweet()
	if tweetinput:get_text():len() > 0 then
		warning:set_text("Please wait...")
	else
		warning:set_text("<span style=\"color:#A00\">Enter movie's name!</span>")
		return
	end
	dlg:update()
	local result, errors = Twitter_SendTweet(twitter_credentials, "Watching " .. vlc.strings.resolve_xml_special_chars(tweetinput:get_text()) .. " with #VLC")
	if not result then
		if not errors then
			warning:set_text("<span style=\"color:#A00\">Unknown error occured</span>")
		else
			warning:set_text("<span style=\"color:#A00\">An error occured: " .. errors .. "</span>")
		end
	else
		warning:set_text("<span style=\"color:#070\">Tweeted successfully!</span>")
	end
end

-- Dialog closed
function close()
    -- Deactivate this extension
    vlc.deactivate()
end