--
-- Really simple HTTP module for VLC
-- Only tested with Twitter module!
-- Written by Maciej Bałuta (December 10, 2010)
--

function http_get(url, add_headers)
	local url = vlc.net.url_parse(url)
	if url.port == 0 then url.port = 80 end
	local fd = vlc.net.connect_tcp(url.host, url.port)
	if fd == -1 then
		vlc.msg.dbg("Could not make a tcp connection to " .. url.host .. ":" .. url.port)
		return nil, nil, "Could not make a tcp connection to " .. url.host .. ":" .. url.port
	end
	local post_headers = ""
	if add_headers then
		for k,v in pairs(add_headers) do
			post_headers = post_headers .. v.field .. ": " .. v.value .. "\r\n"
		end
	end
	vlc.net.send(fd, "GET " .. url.path .. " HTTP/1.1\r\nHost: " .. url.host .. "\r\n" .. post_headers .. "\r\n")
	vlc.msg.dbg("Sent request:\r\n GET " .. url.path .. " HTTP/1.1\r\nHost: " .. url.host .. "\r\n" .. post_headers .. "\r\n")
	local data = ""
	local headers = ""
	local headers_table = { }
	local tmp = ""
	while true do
		tmp = vlc.net.recv(fd)
		headers = headers .. tmp
		tmp = string.match(headers, "\r\n\r\n")
		if tmp then 
			headers_table = {
				content_length = string.match(headers, "Content%-Length: (%d*)\r\n"),
				status = tonumber(string.match(headers, "HTTP/%d%.%d (%d-) .-\r\n"))
			}
			vlc.msg.dbg("Received headers:\r\n" .. headers)
			break
		end
	end
	if headers_table.content_length then
		for i = 1, headers_table.content_length do
			tmp = vlc.net.recv(fd)
			data = data .. tmp
		end
	end
	vlc.msg.dbg("Received data:\r\n" .. data)
	vlc.net.close(fd)
	return data, headers_table
end

function http_post(url, query_string, add_headers)
	local url = vlc.net.url_parse(url)
	if url.port == 0 then url.port = 80 end
	local fd = vlc.net.connect_tcp(url.host, url.port)
	if fd == -1 then
		vlc.msg.dbg("Could not make a tcp connection to " .. url.host .. ":" .. url.port)
		return nil, nil, "Could not make a tcp connection to " .. url.host .. ":" .. url.port
	end
	local post_headers = ""
	if add_headers then
		for k,v in pairs(add_headers) do
			post_headers = post_headers .. v.field .. ": " .. v.value .. "\r\n"
		end
	end
	vlc.net.send(fd, "POST " .. url.path .. " HTTP/1.1\r\nHost: " .. url.host .. "\r\n" .. post_headers .. "\r\n" .. query_string)
	vlc.msg.dbg("Sent request:\r\nPOST " .. url.path .. " HTTP/1.1\r\nHost: " .. url.host .. "\r\n" .. post_headers .. "\r\n" .. query_string)
	local data = ""
	local headers = ""
	local headers_table = { }
	local tmp = ""
	while true do
		tmp = vlc.net.recv(fd)
		headers = headers .. tmp
		tmp = string.match(headers, "\r\n\r\n")
		if tmp then 
			headers_table = {
				content_length = string.match(headers, "Content%-Length: (%d*)\r\n"),
				status = tonumber(string.match(headers, "HTTP/%d%.%d (%d-) .-\r\n"))
			}
			vlc.msg.dbg("Received headers:\r\n" .. headers)
			break
		end
	end
	if headers_table.content_length then
		for i = 1, headers_table.content_length do
			tmp = vlc.net.recv(fd)
			data = data .. tmp
		end
	end
	vlc.msg.dbg("Received data:\r\n" .. data)
	vlc.net.close(fd)
	return data, headers_table
end